@@ -17,7 +17,7 @@
  * Usage
  * [bando@bando cdic-1.0]$ ./cdic [word] or [text]
  * 
- * $Id: cdic.c,v 1.8 2008/01/20 14:27:02 bando Exp $
+ * $Id: cdic.c,v 1.10 2009/08/21 14:56:40 bando Exp $
  * 
  */
 
@@ -44,7 +44,7 @@
 #include <arpa/inet.h>
 
 #define PROG_NAME "cdic"
-#define PROG_VERSION "1.1.2"
+#define PROG_VERSION "1.1.3"
 #define CONF_F ".cdic"
 
 #define SUCCESS 0
@@ -119,29 +119,496 @@ typedef struct _http_request {
 	unsigned int port;
 } http_request;
 
+/*
+ *************************************************
+ *
+ *       socket_get_option()
+ *
+ * 
+ *       socket option    ش.
+ *
+ * 
+ *       1. socket descriptor
+ *       2. SOL_SOCKET
+ *          IPPROTO_IP
+ *          IPPROTO_ICMPV6
+ *          IPPROTO_IPV6
+ *          IPPROTO_TCP
+ *       3.  level   optname
+ *       4.     
+ *
+ * ȯ
+ *        = SUCCESS 
+ *        = FAILURE
+ *
+ *************************************************
+ */
 int socket_get_option (int sock, int level, int optname, void *optval);
+
+/*
+ *************************************************
+ *
+ *       socket_set_option()
+ *
+ * 
+ *       socket option     Ǵ Ѵ.
+ *
+ * 
+ *       1. socket descriptor
+ *       2. SOL_SOCKET
+ *          IPPROTO_IP
+ *          IPPROTO_ICMPV6
+ *          IPPROTO_IPV6
+ *          IPPROTO_TCP
+ *       3.  level   optname
+ *       4.    
+ *
+ * ȯ
+ *        = SUCCESS 
+ *        = FAILURE
+ *
+ *************************************************
+ */
 int socket_set_option (int sock, int level, int optname, const void *optval);
+
+/*
+ *************************************************
+ *
+ *       socket_set_blocking()
+ *
+ * 
+ *       socket  nonblocking  Ѵ.
+ * 
+ *   
+ *       1. socket descriptor
+ *       2. BLOSOCK = blocking socket
+ *          NONSOCK = nonblocking socket
+ * 
+ * ȯ
+ *       fcntl() ȯ 
+ *
+ *************************************************
+ */ 
 int socket_set_blocking (int sock, int block);
-void socket_close (int sock, int l_onoff);
+
+/*
+ *************************************************
+ *
+ *       socket_close()
+ *
+ * 
+ *       socket  close Ѵ.
+ *
+ * 
+ *       1. socket descriptor
+ *       2. LINGERON  = ⺻ ٸ  ʰ  
+ *          LINGEROFF = ⺻ ٸ   (⺻)
+ *
+ * ȯ
+ *       close() ȯ 
+ *
+ *************************************************
+ */
+int socket_close (int sock, int l_onoff);
+
+/*
+ *************************************************
+ *
+ *       tcp_connect()
+ *
+ * 
+ *       tcp  Ѵ.
+ *
+ * 
+ *       1.  
+ *       2.  Ʈ
+ *       3. ִ  ٸ ð(0 = ⺻  Ÿ ƿ ð )
+ *
+ * ȯ
+ *        = socket descriptor
+ *        = -1
+ *
+ *************************************************
+ */
 int tcp_connect (char *server, unsigned int port, unsigned int timeout);
+
+/*
+ *************************************************
+ *
+ *       read_line()
+ *
+ * 
+ *        ũͷκ ڿ  θ оδ.
+ *
+ * 
+ *       1. о ڿ  
+ *       2. б⸦  ڿ ũ
+ *       3.  ũ
+ *
+ * ȯ
+ *        = ̵ 
+ *        = -1
+ *
+ *************************************************
+ */
 int read_line (char *str, size_t size, int fd);
+
+/*
+ *************************************************
+ *
+ *       read_all()
+ *
+ * 
+ *        ũͷκ ü ڿ оδ.
+ *
+ * 
+ *       1.  ũ
+ *
+ * ȯ
+ *        = о ڿ 
+ *        = NULL 
+ *
+ *************************************************
+ */
 char *read_all (int fd);
+
+/*
+ *************************************************
+ *
+ *       write_all()
+ *
+ * 
+ *        ũͿ ڿ Ѵ.
+ *
+ * 
+ *       1.  ũ
+ *       2.  ڿ  
+ *       3.  ڿ 
+ *
+ * ȯ
+ *        =  ڿ 
+ *        = -1
+ *
+ *************************************************
+ */
 int write_all (int fd, const void *buf, size_t count);
+
+/*
+ *************************************************
+ *
+ *       wait_for_input()
+ *
+ * 
+ *       select()   ũ ˻縦 Ѵ.
+ *
+ * 
+ *       1.  ũ
+ *       2. ˻ ð
+ *
+ * ȯ
+ *       select() ȯ
+ *
+ *************************************************
+ */
 int wait_for_input (int fd, unsigned int seconds);
+
+/*
+ *************************************************
+ *
+ *       parse_url()
+ *
+ * 
+ *       URL  ؼ http_request ü
+ *       Ѵ.
+ *
+ * 
+ *       1. URL  
+ *       2. http_request ü 
+ *
+ * ȯ
+ *        = SUCCESS
+ *        = FAILURE
+ *
+ *************************************************
+ */
 int parse_url (const char *url, http_request *hr);
+
+/*
+ *************************************************
+ *
+ *       split()
+ *
+ * 
+ *       ڿ  ڷ Ľ   Ľ̵  ش.
+ *
+ * 
+ *       1. Ľ ڿ  
+ *       2. Ľ 
+ *       3. Ľ̵ ڿ ߿    迭 ȣ
+ *
+ * ȯ
+ *        = Ľ̵  迭 ߿   迭 
+ *        = NULL 
+ *
+ *************************************************
+ */
 char *split (char *str, char split, int ret);
+
+/*
+ *************************************************
+ *
+ *       space_encode()
+ *
+ * 
+ *       鹮ڿ ؼ %20 ȯ Ѵ.
+ *
+ * 
+ *       1.  ڿ  
+ *       2. ȯ ڿ  
+ *       3.  ڿ 
+ *
+ * ȯ
+ *        = ȯ ڿ  
+ *        = NULL 
+ *
+ *************************************************
+ */
 char *space_encode(char *str, char *ret, int ret_len);
+
+/*
+ *************************************************
+ *
+ *       http_send_get()
+ *
+ * 
+ *       http Ǹ Ѵ.
+ *
+ * 
+ *       1.  socketId
+ *       2. http head ü
+ *
+ * ȯ
+ *        = SUCCESS
+ *        = FAILURE
+ *
+ *************************************************
+ */
 int http_send_get (int sock, http_request *hr);
+
+/*
+ *************************************************
+ *
+ *       http_recv_data()
+ *
+ * 
+ *       http  data  Ѵ.
+ *
+ * 
+ *       1.  socketId
+ *
+ * ȯ
+ *        = ŵ data 
+ *        = NULL 
+ *
+ *************************************************
+ */
 char *http_recv_data (int sock);
+
+/*
+ *************************************************
+ *
+ *       http_recv_code()
+ *
+ * 
+ *       http  http code  Ѵ.
+ *
+ * 
+ *       1.  socketId
+ *       2.  ڵ ȣ
+ *
+ * ȯ
+ *        = SUCCESS
+ *        = FAILURE
+ *
+ *************************************************
+ */
 int http_recv_code (int sock, char *replycode);
+
+/*
+ *************************************************
+ *
+ *       str_cut()
+ *
+ * 
+ *       ڿ     ߶󳽴.
+ *
+ * 
+ *       1. ߶ ڿ  
+ *       2.  
+ *       3.  
+ * 
+ * ȯ
+ *        = ߷ ڿ (޵ ڿ ߷ ڿ ȯ Ǿ.)
+ *        = -1
+ *
+ *************************************************
+ */ 
 int str_cut (char *str, char *start, char *end);
+
+/*
+ *************************************************
+ *
+ *       str_toupper()
+ *
+ * 
+ *       ڿ 빮ڷ ٲ۴.
+ *
+ * 
+ *       1. ȯ ڿ  
+ *
+ * ȯ
+ *       ȯ  Ƚ
+ *
+ *************************************************
+ */
 int str_toupper (char *str);
+
+/*
+ *************************************************
+ *
+ *       str_tolower()
+ *
+ * 
+ *       ڿ ҹڷ ٲ۴.
+ *
+ * 
+ *       1. ȯ ڿ  
+ *
+ * ȯ
+ *       ȯ  Ƚ
+ *
+ *************************************************
+ */
 int str_tolower (char *str);
+
+/*
+ *************************************************
+ *
+ *       str_trim()
+ *
+ * 
+ *       ڿ յ  Ѵ.
+ *
+ * 
+ *       1. ȯ ڿ  
+ *
+ * ȯ
+ *       ȯ  Ƚ
+ *
+ *************************************************
+ */
 int str_trim (char *str);
+
+/*
+ *************************************************
+ *
+ *       str_trims()
+ *
+ * 
+ *       ڿ յ    Ѵ.
+ *
+ * 
+ *       1. ȯ ڿ  
+ *
+ * ȯ
+ *        Ƚ
+ *
+ *************************************************
+ */
 int str_trims (char *str);
+
+/*
+ *************************************************
+ *
+ *       str_replace()
+ *
+ * 
+ *       ڿ ġȯ Ѵ.
+ *
+ * 
+ *       1. ȯ ڿ  
+ *       2. ȯ ڿ
+ *       3. ȯ ڿ
+ *
+ * ȯ
+ *       ȯ ڿ Ƚ(޵ ڿ ġȯ  ȯȴ.)
+ *
+ *************************************************
+ */
 int str_replace (char *str, char *now, char *new);
+
+/*
+ *************************************************
+ *
+ *       strip_tag()
+ *
+ * 
+ *       ڿ html ±׸ Ѵ.
+ *
+ * 
+ *       1. ȯ ڿ  
+ *
+ * ȯ
+ *       ŵ html ±װ    = 0
+ *       ŵ html ±װ  = 1
+ *
+ *************************************************
+ */
 int strip_tag (char *str);
 
+// http://www.faqs.org/rfcs/rfc1738.html
+static unsigned char hexchars[] = "0123456789ABCDEF";
+static int url_htoi(char *s);
+
+/*
+ *************************************************
+ *
+ *       url_encode()
+ *
+ * 
+ *       URL encode  Ѵ.
+ *       free()  ʿϴ.
+ *
+ * 
+ *       1. encode  URL
+ *       2. URL 
+ *
+ * ȯ
+ *       ڵ ڿ  
+ *
+ *************************************************
+ */
+char *url_encode(char *s, int len);
+
+/*
+ *************************************************
+ *
+ *       url_decode()
+ *
+ * 
+ *       URL decode  Ѵ.
+ *
+ * 
+ *       1. decode  URL
+ *       2. URL 
+ *
+ * ȯ
+ *       ڵ ڿ  
+ *
+ *************************************************
+ */
+int url_decode(char *str, int len);
+
 #ifdef HAVE_ICONV
 void iconv_error (const char *in_charset, const char *out_charset, iconv_err_type iconv_errno);
 iconv_err_type iconv_convert (char *in_charset, char *out_charset, char *in_str, char **out_str);
@@ -298,7 +765,7 @@ int socket_set_blocking (int sock, int block)
 	return fcntl(sock, F_SETFL, flags);
 }
 
-void socket_close (int sock, int l_onoff)
+int socket_close (int sock, int l_onoff)
 {
 	struct linger linger_v;		
 
@@ -307,7 +774,7 @@ void socket_close (int sock, int l_onoff)
 		linger_v.l_linger = 0x00;
 		socket_set_option(sock, SOL_SOCKET, SO_LINGER, &linger_v);
 	}
-	close(sock);
+	return close(sock);
 }
 
 int tcp_connect (char *server, unsigned int port, unsigned int timeout)
@@ -391,6 +858,7 @@ char *read_all (int fd)
 			break;
 		}
 		else {
+			free(ptr);
 			return (char *)NULL;
 		}
 	}
@@ -532,7 +1000,6 @@ int http_send_get (int sock, http_request *hr)
 			"Connection: close\r\n\r\n"
 			, buf, hr->host, PROG_VERSION
 	);
-
 	if (write_all(sock, hr->rhdr, strlen(hr->rhdr)) < 0)
 	{
 		socket_close(sock, LINGEROFF);
@@ -854,6 +1321,83 @@ int strip_tag (char *str)
 	return (len > strlen(str)) ? 0 : 1;
 }
 
+static int url_htoi(char *s)
+{
+	int value;
+	int c;
+
+	c = s[0];
+	if (isupper(c))
+		c = tolower(c);
+	value = (c >= '0' && c <= '9' ? c - '0' : c - 'a' + 10) * 16;
+
+	c = s[1];
+	if (isupper(c))
+		c = tolower(c);
+	value += c >= '0' && c <= '9' ? c - '0' : c - 'a' + 10;
+
+	return (value);
+}
+
+
+//  free  Ѵ.
+char *url_encode(char *s, int len)
+{
+	register int x, y;
+	unsigned char *str;
+
+	str = (unsigned char *) malloc(3 * strlen(s) + 1);
+	for (x = 0, y = 0; len--; x++, y++) {
+		str[y] = (unsigned char) s[x];
+		if (str[y] == ' ') {
+			str[y] = '+';
+#ifndef CHARSET_EBCDIC
+		} else if ((str[y] < '0' && str[y] != '-' && str[y] != '.') ||
+				(str[y] < 'A' && str[y] > '9') ||
+				(str[y] > 'Z' && str[y] < 'a' && str[y] != '_') ||
+				(str[y] > 'z')) {
+			str[y++] = '%';
+			str[y++] = hexchars[(unsigned char) s[x] >> 4];
+			str[y] = hexchars[(unsigned char) s[x] & 15];
+		}
+#else /*CHARSET_EBCDIC*/
+		} else if (!isalnum(str[y]) && strchr("_-.", str[y]) == NULL) {
+			/* Allow only alphanumeric chars and '_', '-', '.'; escape the rest */
+			str[y++] = '%';
+			str[y++] = hexchars[os_toascii[(unsigned char) s[x]] >> 4];
+			str[y] = hexchars[os_toascii[(unsigned char) s[x]] & 0x0F];
+		}
+#endif /*CHARSET_EBCDIC*/
+	}
+	str[y] = '\0';
+	return ((char *) str);
+}
+
+int url_decode(char *str, int len)
+{
+	char *dest = str;
+	char *data = str;
+
+	while (len--) {
+		if (*data == '+')
+			*dest = ' ';
+		else if (*data == '%' && len >= 2 && isxdigit((int) *(data + 1)) && isxdigit((int) *(data + 2))) {
+#ifndef CHARSET_EBCDIC
+			*dest = (char) url_htoi(data + 1);
+#else
+			*dest = os_toebcdic[(char) url_htoi(data + 1)];
+#endif
+			data += 2;
+			len -= 2;
+		} else
+			*dest = *data;
+		data++;
+		dest++;
+	}
+	*dest = '\0';
+	return dest - str;
+}
+
 #ifdef HAVE_ICONV
 void iconv_error (const char *in_charset, const char *out_charset, iconv_err_type iconv_errno)
 {
@@ -1205,9 +1749,11 @@ void trans_google (char *str, config *cf)
 	char *ptr = NULL;
 	char buf[BUFSIZE];
 	char *recved = NULL, *lang_c;
-	char url[] = "http://translate.google.com/translate_t?langpair=%s&ie=EUC-KR&text=%s";
+	char url[] = "http://www.google.com/uds/Gtranslate?callback=google.language.callbacks.cdic&context=&langpair=%s&q=%s&v=1.0";
+	char *sp = NULL, *ep = NULL;
 #ifdef HAVE_ICONV
 	char *out_str = NULL;
+	char *url_str = NULL;
 #endif
 
 	ptr = strchr(str, ' ');
@@ -1219,22 +1765,32 @@ void trans_google (char *str, config *cf)
 			break;
 		}
 	}
-
+#ifdef HAVE_ICONV
+			iconv_error(CDSET, cf->charset, iconv_convert (CDSET, "UTF-8", str, &out_str));
+			url_str = url_encode(out_str, strlen(out_str));
+			free(out_str);
+#endif
 	memset(buf, 0x0, sizeof(buf));
-	snprintf(buf, sizeof(buf) - 1, url, lang_c, str);
+	snprintf(buf, sizeof(buf) - 1, url, lang_c, url_str);
 
 	if ((recved = http_request_get(buf))) {
 #ifdef HAVE_ICONV
 		str_toupper(cf->charset);
-		if (strcmp(CDSET, cf->charset) != 0) {
-			iconv_error(CDSET, cf->charset, iconv_convert (CDSET, cf->charset, recved, &out_str));
-			free(recved);
-			recved = out_str;
-		}
-#endif
-		str_cut(recved, "<textarea name=utrans wrap=SOFT dir=\"ltr\" rows=5 id=suggestion>", "</textarea>");
-		printf("%s\n", recved);
+		iconv_error("UTF-8", cf->charset, iconv_convert ("UTF-8", cf->charset, recved, &out_str));
 		free(recved);
+		recved = out_str;
+#endif
+		str_replace (recved, "\\u0026#39;", "'");
+		str_replace (recved, "\\u0026quot;", "\"");
+		sp = strstr(recved, "translatedText") + strlen("translatedText") + 3;
+		ep = strstr(sp, "}");
+		*(ep - 1) = '\0';
+		printf("%s\n", sp);
+		free(recved);
+#ifdef HAVE_ICONV
+		free(url_str);
+#endif
+
 	}
 }
 
