@@ -17,7 +17,7 @@
  * Usage
  * [bando@bando cdic-1.0]$ ./cdic [word] or [text]
  * 
- * $Id: cdic.utf8.c,v 1.2 2008/01/20 14:27:02 bando Exp $
+ * $Id: cdic.utf8.c,v 1.4 2009/08/21 14:56:50 bando Exp $
  * 
  */
 
@@ -44,7 +44,7 @@
 #include <arpa/inet.h>
 
 #define PROG_NAME "cdic"
-#define PROG_VERSION "1.1.2"
+#define PROG_VERSION "1.1.3"
 #define CONF_F ".cdic"
 
 #define SUCCESS 0
@@ -119,29 +119,496 @@ typedef struct _http_request {
 	unsigned int port;
 } http_request;
 
+/*
+ *************************************************
+ *
+ *       socket_get_option()
+ *
+ * 설명
+ *       socket option에 대한 현재 값을 돌려준다.
+ *
+ * 인자
+ *       1. socket descriptor
+ *       2. SOL_SOCKET
+ *          IPPROTO_IP
+ *          IPPROTO_ICMPV6
+ *          IPPROTO_IPV6
+ *          IPPROTO_TCP
+ *       3. 각 level 에 속한 optname
+ *       4. 가져올 값을 저장할 변수 
+ *
+ * 반환값
+ *       성공 = SUCCESS 
+ *       실패 = FAILURE
+ *
+ *************************************************
+ */
 int socket_get_option (int sock, int level, int optname, void *optval);
+
+/*
+ *************************************************
+ *
+ *       socket_set_option()
+ *
+ * 설명
+ *       socket option에 대한 현재 값을 설정 또는 변경한다.
+ *
+ * 인자
+ *       1. socket descriptor
+ *       2. SOL_SOCKET
+ *          IPPROTO_IP
+ *          IPPROTO_ICMPV6
+ *          IPPROTO_IPV6
+ *          IPPROTO_TCP
+ *       3. 각 level 에 속한 optname
+ *       4. 변경할 값을 저장한 변수
+ *
+ * 반환값
+ *       성공 = SUCCESS 
+ *       실패 = FAILURE
+ *
+ *************************************************
+ */
 int socket_set_option (int sock, int level, int optname, const void *optval);
+
+/*
+ *************************************************
+ *
+ *       socket_set_blocking()
+ *
+ * 설명
+ *       socket 을 nonblocking 으로 설정한다.
+ * 
+ * 인자  
+ *       1. socket descriptor
+ *       2. BLOSOCK = blocking socket
+ *          NONSOCK = nonblocking socket
+ * 
+ * 반환값
+ *       fcntl() 반환 값
+ *
+ *************************************************
+ */ 
 int socket_set_blocking (int sock, int block);
-void socket_close (int sock, int l_onoff);
+
+/*
+ *************************************************
+ *
+ *       socket_close()
+ *
+ * 설명
+ *       socket 을 close 한다.
+ *
+ * 인자
+ *       1. socket descriptor
+ *       2. LINGERON  = 기본적인 기다림을 가지지 않고 곧장 종료
+ *          LINGEROFF = 기본적인 기다림을 가진후 종료 (기본값)
+ *
+ * 반환값
+ *       close() 반환 값
+ *
+ *************************************************
+ */
+int socket_close (int sock, int l_onoff);
+
+/*
+ *************************************************
+ *
+ *       tcp_connect()
+ *
+ * 설명
+ *       tcp 연결을 수행한다.
+ *
+ * 인자
+ *       1. 목적지 서버
+ *       2. 목적지 포트
+ *       3. 최대 연결 기다림 시간(0 = 기본 소켓 타임 아웃 시간 설정)
+ *
+ * 반환값
+ *       성공 = socket descriptor
+ *       실패 = -1
+ *
+ *************************************************
+ */
 int tcp_connect (char *server, unsigned int port, unsigned int timeout);
+
+/*
+ *************************************************
+ *
+ *       read_line()
+ *
+ * 설명
+ *       파일 디스크립터로부터 문자열을 한 라인만 읽어들인다.
+ *
+ * 인자
+ *       1. 읽어들인 문자열을 저장할 포인터
+ *       2. 읽기를 수행할 문자열의 크기
+ *       3. 파일 디스크립터
+ *
+ * 반환값
+ *       성공 = 이동된 포인터
+ *       실패 = -1
+ *
+ *************************************************
+ */
 int read_line (char *str, size_t size, int fd);
+
+/*
+ *************************************************
+ *
+ *       read_all()
+ *
+ * 설명
+ *       파일 디스크립터로부터 전체 문자열을 읽어들인다.
+ *
+ * 인자
+ *       1. 파일 디스크립터
+ *
+ * 반환값
+ *       성공 = 읽어들인 문자열의 포인터
+ *       실패 = NULL 포인터
+ *
+ *************************************************
+ */
 char *read_all (int fd);
+
+/*
+ *************************************************
+ *
+ *       write_all()
+ *
+ * 설명
+ *       파일 디스크립터에 문자열을 기록한다.
+ *
+ * 인자
+ *       1. 파일 디스크립터
+ *       2. 기록할 문자열에 대한 포인터
+ *       3. 기록할 문자열의 길이
+ *
+ * 반환값
+ *       성공 = 기록한 문자열의 길이
+ *       실패 = -1
+ *
+ *************************************************
+ */
 int write_all (int fd, const void *buf, size_t count);
+
+/*
+ *************************************************
+ *
+ *       wait_for_input()
+ *
+ * 설명
+ *       select() 를 통한 디스크립터 검사를 수행한다.
+ *
+ * 인자
+ *       1. 파일 디스크립터
+ *       2. 검사할 시간
+ *
+ * 반환값
+ *       select() 반환값
+ *
+ *************************************************
+ */
 int wait_for_input (int fd, unsigned int seconds);
+
+/*
+ *************************************************
+ *
+ *       parse_url()
+ *
+ * 설명
+ *       URL 구분을 수행해서 http_request 구조체에
+ *       저장한다.
+ *
+ * 인자
+ *       1. URL에 대한 포인터
+ *       2. http_request 구조체 포인터
+ *
+ * 반환값
+ *       성공 = SUCCESS
+ *       실패 = FAILURE
+ *
+ *************************************************
+ */
 int parse_url (const char *url, http_request *hr);
+
+/*
+ *************************************************
+ *
+ *       split()
+ *
+ * 설명
+ *       문자열에 대해 구분자로 파싱후 지정한 순서의 파싱된 값을 돌려준다.
+ *
+ * 인자
+ *       1. 파싱할 문자열에 대한 포인터
+ *       2. 파싱 구분자
+ *       3. 파싱된 문자열들 중에서 돌려 받을 문자 배열 번호
+ *
+ * 반환값
+ *       성공 = 파싱된 문자 배열들 중에서 지정한 문자 배열의 포인터
+ *       에러 = NULL 포인터
+ *
+ *************************************************
+ */
 char *split (char *str, char split, int ret);
+
+/*
+ *************************************************
+ *
+ *       space_encode()
+ *
+ * 설명
+ *       공백문자에 대해서 %20 변환을 수행한다.
+ *
+ * 인자
+ *       1. 원본 문자열에 대한 포인터
+ *       2. 변환된 문자열이 저장될 포인터
+ *       3. 원본 문자열의 길이
+ *
+ * 반환값
+ *       성공 = 변환된 문자열이 저장된 포인터
+ *       에러 = NULL 포인터
+ *
+ *************************************************
+ */
 char *space_encode(char *str, char *ret, int ret_len);
+
+/*
+ *************************************************
+ *
+ *       http_send_get()
+ *
+ * 설명
+ *       http 질의를 수행한다.
+ *
+ * 인자
+ *       1. 연결된 socketId
+ *       2. http head 구조체
+ *
+ * 반환값
+ *       성공 = SUCCESS
+ *       에러 = FAILURE
+ *
+ *************************************************
+ */
 int http_send_get (int sock, http_request *hr);
+
+/*
+ *************************************************
+ *
+ *       http_recv_data()
+ *
+ * 설명
+ *       http 수행후 data를 수신을 수행한다.
+ *
+ * 인자
+ *       1. 연결된 socketId
+ *
+ * 반환값
+ *       성공 = 수신된 data의 포인터
+ *       에러 = NULL 포인터
+ *
+ *************************************************
+ */
 char *http_recv_data (int sock);
+
+/*
+ *************************************************
+ *
+ *       http_recv_code()
+ *
+ * 설명
+ *       http 수행후 http code 를 비교한다.
+ *
+ * 인자
+ *       1. 연결된 socketId
+ *       2. 비교할 코드 번호
+ *
+ * 반환값
+ *       성공 = SUCCESS
+ *       에러 = FAILURE
+ *
+ *************************************************
+ */
 int http_recv_code (int sock, char *replycode);
+
+/*
+ *************************************************
+ *
+ *       str_cut()
+ *
+ * 설명
+ *       문자열을 지정한 시작점과 끝점을 기준으로 잘라낸다.
+ *
+ * 인자
+ *       1. 잘라낼 문자열에 대한 포인터
+ *       2. 시작점 문자
+ *       3. 끝점 문자
+ * 
+ * 반환값
+ *       성공 = 잘려진 문자열의 길이(전달된 문자열은 잘려진 문자열로 변환 되어진다.)
+ *       에러 = -1
+ *
+ *************************************************
+ */ 
 int str_cut (char *str, char *start, char *end);
+
+/*
+ *************************************************
+ *
+ *       str_toupper()
+ *
+ * 설명
+ *       문자열을 대문자로 바꾼다.
+ *
+ * 인자
+ *       1. 변환할 문자열에 대한 포인터
+ *
+ * 반환값
+ *       변환이 수행된 횟수
+ *
+ *************************************************
+ */
 int str_toupper (char *str);
+
+/*
+ *************************************************
+ *
+ *       str_tolower()
+ *
+ * 설명
+ *       문자열을 소문자로 바꾼다.
+ *
+ * 인자
+ *       1. 변환할 문자열에 대한 포인터
+ *
+ * 반환값
+ *       변환이 수행된 횟수
+ *
+ *************************************************
+ */
 int str_tolower (char *str);
+
+/*
+ *************************************************
+ *
+ *       str_trim()
+ *
+ * 설명
+ *       문자열의 앞뒤 공백을 제거한다.
+ *
+ * 인자
+ *       1. 변환할 문자열에 대한 포인터
+ *
+ * 반환값
+ *       변환이 수행된 횟수
+ *
+ *************************************************
+ */
 int str_trim (char *str);
+
+/*
+ *************************************************
+ *
+ *       str_trims()
+ *
+ * 설명
+ *       문자열의 앞뒤 공백을 개행을 구분으로 제거한다.
+ *
+ * 인자
+ *       1. 변환할 문자열에 대한 포인터
+ *
+ * 반환값
+ *       개행의 횟수
+ *
+ *************************************************
+ */
 int str_trims (char *str);
+
+/*
+ *************************************************
+ *
+ *       str_replace()
+ *
+ * 설명
+ *       문자열을 치환 한다.
+ *
+ * 인자
+ *       1. 변환할 문자열에 대한 포인터
+ *       2. 변환전 문자열
+ *       3. 변환후 문자열
+ *
+ * 반환값
+ *       변환전 문자열의 횟수(전달된 문자열은 치환된 값으로 변환된다.)
+ *
+ *************************************************
+ */
 int str_replace (char *str, char *now, char *new);
+
+/*
+ *************************************************
+ *
+ *       strip_tag()
+ *
+ * 설명
+ *       문자열에서 html 태그를 제거한다.
+ *
+ * 인자
+ *       1. 변환할 문자열에 대한 포인터
+ *
+ * 반환값
+ *       제거된 html 태그가 존재시   = 0
+ *       제거된 html 태그가 미존재시 = 1
+ *
+ *************************************************
+ */
 int strip_tag (char *str);
 
+// http://www.faqs.org/rfcs/rfc1738.html
+static unsigned char hexchars[] = "0123456789ABCDEF";
+static int url_htoi(char *s);
+
+/*
+ *************************************************
+ *
+ *       url_encode()
+ *
+ * 설명
+ *       URL encode 를 수행한다.
+ *       free() 수행이 필요하다.
+ *
+ * 인자
+ *       1. encode 될 URL
+ *       2. URL 길이
+ *
+ * 반환값
+ *       인코딩된 문자열에 대한 포인터
+ *
+ *************************************************
+ */
+char *url_encode(char *s, int len);
+
+/*
+ *************************************************
+ *
+ *       url_decode()
+ *
+ * 설명
+ *       URL decode 를 수행한다.
+ *
+ * 인자
+ *       1. decode 될 URL
+ *       2. URL 길이
+ *
+ * 반환값
+ *       디코딩된 문자열에 대한 포인터
+ *
+ *************************************************
+ */
+int url_decode(char *str, int len);
+
 #ifdef HAVE_ICONV
 void iconv_error (const char *in_charset, const char *out_charset, iconv_err_type iconv_errno);
 iconv_err_type iconv_convert (char *in_charset, char *out_charset, char *in_str, char **out_str);
@@ -298,7 +765,7 @@ int socket_set_blocking (int sock, int block)
 	return fcntl(sock, F_SETFL, flags);
 }
 
-void socket_close (int sock, int l_onoff)
+int socket_close (int sock, int l_onoff)
 {
 	struct linger linger_v;		
 
@@ -307,7 +774,7 @@ void socket_close (int sock, int l_onoff)
 		linger_v.l_linger = 0x00;
 		socket_set_option(sock, SOL_SOCKET, SO_LINGER, &linger_v);
 	}
-	close(sock);
+	return close(sock);
 }
 
 int tcp_connect (char *server, unsigned int port, unsigned int timeout)
@@ -391,6 +858,7 @@ char *read_all (int fd)
 			break;
 		}
 		else {
+			free(ptr);
 			return (char *)NULL;
 		}
 	}
@@ -532,7 +1000,6 @@ int http_send_get (int sock, http_request *hr)
 			"Connection: close\r\n\r\n"
 			, buf, hr->host, PROG_VERSION
 	);
-
 	if (write_all(sock, hr->rhdr, strlen(hr->rhdr)) < 0)
 	{
 		socket_close(sock, LINGEROFF);
@@ -854,6 +1321,83 @@ int strip_tag (char *str)
 	return (len > strlen(str)) ? 0 : 1;
 }
 
+static int url_htoi(char *s)
+{
+	int value;
+	int c;
+
+	c = s[0];
+	if (isupper(c))
+		c = tolower(c);
+	value = (c >= '0' && c <= '9' ? c - '0' : c - 'a' + 10) * 16;
+
+	c = s[1];
+	if (isupper(c))
+		c = tolower(c);
+	value += c >= '0' && c <= '9' ? c - '0' : c - 'a' + 10;
+
+	return (value);
+}
+
+
+// 받은후 free 해줘야 한다.
+char *url_encode(char *s, int len)
+{
+	register int x, y;
+	unsigned char *str;
+
+	str = (unsigned char *) malloc(3 * strlen(s) + 1);
+	for (x = 0, y = 0; len--; x++, y++) {
+		str[y] = (unsigned char) s[x];
+		if (str[y] == ' ') {
+			str[y] = '+';
+#ifndef CHARSET_EBCDIC
+		} else if ((str[y] < '0' && str[y] != '-' && str[y] != '.') ||
+				(str[y] < 'A' && str[y] > '9') ||
+				(str[y] > 'Z' && str[y] < 'a' && str[y] != '_') ||
+				(str[y] > 'z')) {
+			str[y++] = '%';
+			str[y++] = hexchars[(unsigned char) s[x] >> 4];
+			str[y] = hexchars[(unsigned char) s[x] & 15];
+		}
+#else /*CHARSET_EBCDIC*/
+		} else if (!isalnum(str[y]) && strchr("_-.", str[y]) == NULL) {
+			/* Allow only alphanumeric chars and '_', '-', '.'; escape the rest */
+			str[y++] = '%';
+			str[y++] = hexchars[os_toascii[(unsigned char) s[x]] >> 4];
+			str[y] = hexchars[os_toascii[(unsigned char) s[x]] & 0x0F];
+		}
+#endif /*CHARSET_EBCDIC*/
+	}
+	str[y] = '\0';
+	return ((char *) str);
+}
+
+int url_decode(char *str, int len)
+{
+	char *dest = str;
+	char *data = str;
+
+	while (len--) {
+		if (*data == '+')
+			*dest = ' ';
+		else if (*data == '%' && len >= 2 && isxdigit((int) *(data + 1)) && isxdigit((int) *(data + 2))) {
+#ifndef CHARSET_EBCDIC
+			*dest = (char) url_htoi(data + 1);
+#else
+			*dest = os_toebcdic[(char) url_htoi(data + 1)];
+#endif
+			data += 2;
+			len -= 2;
+		} else
+			*dest = *data;
+		data++;
+		dest++;
+	}
+	*dest = '\0';
+	return dest - str;
+}
+
 #ifdef HAVE_ICONV
 void iconv_error (const char *in_charset, const char *out_charset, iconv_err_type iconv_errno)
 {
@@ -1205,9 +1749,11 @@ void trans_google (char *str, config *cf)
 	char *ptr = NULL;
 	char buf[BUFSIZE];
 	char *recved = NULL, *lang_c;
-	char url[] = "http://translate.google.com/translate_t?langpair=%s&ie=EUC-KR&text=%s";
+	char url[] = "http://www.google.com/uds/Gtranslate?callback=google.language.callbacks.cdic&context=&langpair=%s&q=%s&v=1.0";
+	char *sp = NULL, *ep = NULL;
 #ifdef HAVE_ICONV
 	char *out_str = NULL;
+	char *url_str = NULL;
 #endif
 
 	ptr = strchr(str, ' ');
@@ -1219,22 +1765,32 @@ void trans_google (char *str, config *cf)
 			break;
 		}
 	}
-
+#ifdef HAVE_ICONV
+			iconv_error(CDSET, cf->charset, iconv_convert (CDSET, "UTF-8", str, &out_str));
+			url_str = url_encode(out_str, strlen(out_str));
+			free(out_str);
+#endif
 	memset(buf, 0x0, sizeof(buf));
-	snprintf(buf, sizeof(buf) - 1, url, lang_c, str);
+	snprintf(buf, sizeof(buf) - 1, url, lang_c, url_str);
 
 	if ((recved = http_request_get(buf))) {
 #ifdef HAVE_ICONV
 		str_toupper(cf->charset);
-		if (strcmp(CDSET, cf->charset) != 0) {
-			iconv_error(CDSET, cf->charset, iconv_convert (CDSET, cf->charset, recved, &out_str));
-			free(recved);
-			recved = out_str;
-		}
-#endif
-		str_cut(recved, "<textarea name=utrans wrap=SOFT dir=\"ltr\" rows=5 id=suggestion>", "</textarea>");
-		printf("%s\n", recved);
+		iconv_error("UTF-8", cf->charset, iconv_convert ("UTF-8", cf->charset, recved, &out_str));
 		free(recved);
+		recved = out_str;
+#endif
+		str_replace (recved, "\\u0026#39;", "'");
+		str_replace (recved, "\\u0026quot;", "\"");
+		sp = strstr(recved, "translatedText") + strlen("translatedText") + 3;
+		ep = strstr(sp, "}");
+		*(ep - 1) = '\0';
+		printf("%s\n", sp);
+		free(recved);
+#ifdef HAVE_ICONV
+		free(url_str);
+#endif
+
 	}
 }
 
